--[[
	Author: Elsia
	BattleObjectivesMover, moves the Battle Objectives Frame.
--]]

BattleObjectivesMover = AceLibrary("AceAddon-2.0"):new("AceDB-2.0", "AceConsole-2.0")

local Default = {
	locked = true,
	pos	= { x = WorldStateAlwaysUpFrame:GetLeft(), y = WorldStateAlwaysUpFrame:GetTop()},
}

function BattleObjectivesMover:OnInitialize()

	self.f = WorldStateAlwaysUpFrame

	local Options = {
		type = "group",
		args = {
	        lock = {
	            name = "Lock",
	            type = "toggle",
	            desc = "Lock/Unlock the battle objectives frame.",
	            get = function() return self.db.profile.locked end,
	            set = function( v ) self.db.profile.locked = v if(v) then
		if self.master  then self.master:EnableMouse(false)
		self:HideDragger() self:SetPosition() end else self:ShowDragger()
		self.master:EnableMouse(true) end end,
	            map = {[false] = "Unlocked", [true] = "Locked"},
	            guiNameIsMap = true,
            },
		}
	}
	
	self:RegisterDB("BattleObjectivesMoverDB")
	self:RegisterDefaults('profile', Default)
	self:RegisterChatCommand( {"/battleobjectivesmover"}, Options )
	self:RegisterChatCommand( {"/bom"}, Options )
end

function BattleObjectivesMover:ShowDragger()

	if not self.master then    -- Only creates frame when dragging is enabled.
	   self:CreateFrame()
	end

	self:ReparentFrame()
    self:SetPosition()

	self.master:SetBackdropBorderColor(TOOLTIP_DEFAULT_COLOR.r, TOOLTIP_DEFAULT_COLOR.g, TOOLTIP_DEFAULT_COLOR.b, 0.5)
	self.master:SetBackdropColor(TOOLTIP_DEFAULT_BACKGROUND_COLOR.r, TOOLTIP_DEFAULT_BACKGROUND_COLOR.g, TOOLTIP_DEFAULT_BACKGROUND_COLOR.b, 0.5)
	self.master.label:Show()
end

function BattleObjectivesMover:HideDragger()
	self.master:SetBackdropBorderColor(0,0,0,0)
	self.master:SetBackdropColor(0,0,0,0)
	self.master.label:Hide()
end

function BattleObjectivesMover:ReparentFrame()
	self.f:ClearAllPoints()
	self.f:SetParent(self.master)
	local s = self.f:GetEffectiveScale()
	self.f:ClearAllPoints()
	self.f:SetPoint("TOPLEFT", self.master, "TOPLEFT", 0, 0)
	
end

function BattleObjectivesMover:CreateFrame()
	self.master = CreateFrame("Frame", "BattleObjectivesMoverFrame", UIParent)

	self.master:SetMovable(true)

	self.master:RegisterForDrag("LeftButton")
	self.master:SetScript("OnDragStart", function() if not self.db.profile.locked then self["master"]:StartMoving() end end)
	self.master:SetScript("OnDragStop", function() self["master"]:StopMovingOrSizing() self:SavePosition() end)
		
	self.master.label = self.master:CreateFontString(nil, "ARTWORK")
	self.master.label:SetFontObject(TextStatusBarText)
    
	self.master:SetWidth( 150 )
	self.master:SetHeight( 75 )

	self.master:SetBackdrop({
			bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 8,
			edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", 
			edgeSize = 16,
			insets = {left = 4, right = 4, top = 4, bottom = 4},
		})
	self.master:SetBackdropBorderColor(TOOLTIP_DEFAULT_COLOR.r, TOOLTIP_DEFAULT_COLOR.g, TOOLTIP_DEFAULT_COLOR.b, 0.5)
	self.master:SetBackdropColor(TOOLTIP_DEFAULT_BACKGROUND_COLOR.r, TOOLTIP_DEFAULT_BACKGROUND_COLOR.g, TOOLTIP_DEFAULT_BACKGROUND_COLOR.b, 0.5)

	self.master.label:SetJustifyH("CENTER")
	local labelFont, osize, oflags = GameFontHighlightSmall:GetFont()
	self.master.label:SetFont( labelFont, osize, oflags )
	self.master.label:ClearAllPoints()
	self.master.label:SetPoint("CENTER", self.master, "CENTER",0,0)
	self.master.label:SetTextColor( 1,1,1 )
	self.master.label:SetText( "Moveable Battle Objectives" )
	
	self.master:Show()
end

function BattleObjectivesMover:OnEnable()

	 if not self.db.profile.locked then
	    self:ShowDragger()
	    self.master:EnableMouse(true)
	 end
	 self:SetPosition()
end

function BattleObjectivesMover:SavePosition()

	local x, y = self.master:GetLeft(), self.master:GetTop()
	local s = self.master:GetEffectiveScale()
	
	self.db.profile.pos.x = x * s
	self.db.profile.pos.y = y * s

	self:SetPosition() -- Not sure if this is needed
end

function BattleObjectivesMover:SetPosition()

	if self.db.profile.pos.x then
		local x = self.db.profile.pos.x
		local y = self.db.profile.pos.y

		local s = self.f:GetEffectiveScale()

		if self.db.profile.locked then
			self.f:ClearAllPoints()
			self.f:SetParent(UIParent)
			self.f:ClearAllPoints()
			self.f:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", x / s, y / s) -- Seems like the frame is forcing "TOP" :P
		else
			self.master:ClearAllPoints()
			self.master:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", x / s, y / s)
		end
	end
end
